@@ -1,5 +1,5 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2014-2020 Rumma & Ko Ltd
+# Copyright 2014-2022 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 """This is the main module of Lino Noi.
 
@@ -11,10 +11,10 @@
 
 """
 
-from lino_noi.setup_info import SETUP_INFO
+from .setup_info import SETUP_INFO
 
 __version__ = SETUP_INFO['version']
 
-srcref_url = 'https://gitlab.com/lino-framework/noi/blob/master/%s'
-intersphinx_urls = dict(docs="https://noi.lino-framework.org")
-doc_trees = ['docs']
+# srcref_url = 'https://gitlab.com/lino-framework/noi/blob/master/%s'
+# intersphinx_urls = dict(docs="https://noi.lino-framework.org")
+# doc_trees = ['docs']
