@@ -8,6 +8,8 @@ Defines a customized :class:`TicketDetail`.
 
 """
 
+import logging ; logger = logging.getLogger(__name__)
+
 from lino.api import _
 from lino.mixins.sequenced import Hierarchical
 from lino.modlib.users.mixins import Assignable
@@ -28,28 +30,9 @@ class Site(Site):
         abstract = dd.is_abstract_model(__name__, 'Site')
 
     def get_change_observers(self, ar=None):
-        # action = ar.bound_action.action if ar.bound_action else None
-        # if ar is not None and isinstance(action, CreateRow) and issubclass(ar.actor.model, Ticket):
-        if False:
-            # 20210928 The idea was to notify only triagers when a new site was
-            # created for a team. We can throw this away, seems that this made
-            # some sense a long time ago when we didn't yet have teams.
-            subs = rt.models.groups.Group.objects.filter(site=ar.selected_rows[-1].site)
-            #subs = rt.models.tickets.Subscription.objects.filter(site=ar.selected_rows[-1].site)
-            for s in subs.members.all():
-                for (u, um) in [(u, u.mail_mode) for u in [sub.user for sub in s]
-                                if (u.user_type and u.user_type.has_required_roles([Triager])
-                                    and u != ar.get_user())]:
-                    yield (u, um)
-        else:
-            #for s in rt.models.tickets.Subscription.objects.filter(site=self):
-            for s in rt.models.groups.Group.objects.filter(site=self):
-                for sub in s.members.all():
-                    yield (sub.user, sub.user.mail_mode)
-
-    #def after_ui_create(self, ar):
-    #    super(Site, self).after_ui_create(ar)
-    #    rt.models.tickets.Subscription.objects.create(user=ar.get_user(), site=self)
+        for s in rt.models.groups.Group.objects.filter(site=self):
+            for sub in s.members.all():
+                yield (sub.user, sub.user.mail_mode)
 
 
 class Ticket(Ticket, Assignable, SummarizedFromSession, Hierarchical, ElasticSearchable):
@@ -172,39 +155,22 @@ class Ticket(Ticket, Assignable, SummarizedFromSession, Hierarchical, ElasticSea
         }
     )]
 
-    def assigned_to_changed(self, ar):
-        """Add a star and send notification of Assignment"""
-        # self.add_change_watcher(self.assigned_to)
+    if dd.is_installed('notify'):
+        def assigned_to_changed(self, ar):
+            """Send notification of assignment"""
 
-        if (self.assigned_to is not None and
-                self.assigned_to != ar.user and
-                dd.is_installed('notify')):
-            ctx = dict(user=ar.user, what=ar.obj2memo(self))
-            def msg(user, mm):
-                subject = _("{user} has assigned you to ticket: {what}").format(**ctx)
-                return (subject , tostring(E.span(subject)))
+            if (self.assigned_to is not None and self.assigned_to != ar.user):
+                ctx = dict(user=ar.user, what=self.obj2memo())
+                def msg(user, mm):
+                    subject = _("{user} has assigned you to ticket: {what}").format(**ctx)
+                    return (subject , tostring(E.span(subject)))
 
-            mt = rt.models.notify.MessageTypes.tickets
+                mt = rt.models.notify.MessageTypes.tickets
 
-            rt.models.notify.Message.emit_notification(
-                ar, self, mt, msg,
-                [(self.assigned_to, self.assigned_to.mail_mode)]
-            )
-    # def end_user_changed(self, ar):
-    #     """Add a star"""
-    #     self.add_change_watcher(self.end_user)
-
-    # def user_changed(self, ar):
-    #     """Add a star"""
-    #     self.add_change_watcher(self.user)
-
-    # def after_ui_create(self, ar):
-    #     # print("Create")
-    #     # self.site_changed(ar)
-    #     # self.assigned_to_changed(ar)
-    #     # self.end_user_changed(ar)
-    #     # self.user_changed(ar)
-    #     super(Ticket, self).after_ui_create(ar)
+                rt.models.notify.Message.emit_notification(
+                    ar, self, mt, msg,
+                    [(self.assigned_to, self.assigned_to.mail_mode)]
+                )
 
     # show_commits = dd.ShowSlaveTable('github.CommitsByTicket')
     show_changes = dd.ShowSlaveTable('changes.ChangesByMaster')
@@ -216,13 +182,13 @@ class Ticket(Ticket, Assignable, SummarizedFromSession, Hierarchical, ElasticSea
         ctx = dict(user=ar.user, what=str(self))
         if cw is None:
             # return _("{user} submitted ticket {what}").format(**ctx)
-            return 
+            return
         if len(list(cw.get_updates())) == 0:
             return
         return _("{user} modified {what}").format(**ctx)
 
     def get_change_body(self, ar, cw):
-        ctx = dict(user=ar.user, what=ar.obj2memo(self))
+        ctx = dict(user=ar.user, what=self.obj2memo())
         if cw is None:
             elems = [E.p(
                 _("{user} submitted ticket {what}").format(**ctx), ".")]
@@ -298,7 +264,7 @@ class TicketDetail(TicketDetail):
     overview
     """
     general2 = """
-    site end_user ticket_type
+    order end_user ticket_type
     triager_panel
     priority:10 planned_time deadline
     SUMMARY_FIELDS
@@ -307,11 +273,12 @@ class TicketDetail(TicketDetail):
 
     triager_panel = dd.Panel(
     """
-    quick_assign_to private:10
+    site quick_assign_to private:10
     """, required_roles=dd.login_required(Triager))
 
     general3 = """
     workflow_buttons
+    comment
     comments.CommentsByRFC:30
     """
 
@@ -455,3 +422,60 @@ class TicketsByParent(Tickets):
     master_key = 'parent'
     column_names = "priority id summary:50 quick_assign_to #ticket_type #workflow_buttons *"
     details_of_master_template = _("Children of %(master)s")
+
+
+from lino.modlib.checkdata.choicelists import Checker
+
+class TicketOrderChecker(Checker):
+    # Can be removed when all production sites have upgraded to lino-noi>=22.12
+    verbose_name = _("Fill the new 'order' field.")
+    model = Ticket
+
+    def get_checkdata_problems(self, obj, fix=False):
+        if obj.site_id is None:
+            return
+        if obj.site.ref is None:
+            return
+        # if obj.site.company == settings.SITE.site_config.site_company:
+        #     return
+
+        cust = obj.site.ref.startswith("cust.")
+        if not cust:
+            if obj.order_id is not None:
+                yield (True, _("Order should be empty (not a customer project)"))
+                if fix:
+                    obj.order = None
+                    obj.full_clean()
+                    obj.save()
+                    # logger.info("Removed order because its not a customer project", obj)
+            return
+
+        if obj.order_id and obj.order.ref == obj.site.ref:
+            return
+        yield (True, _("Must populate order from project"))
+        if fix:
+            # Not tested. We'll just get it work after the upgrade
+            Subscription = rt.models.subscriptions.Subscription
+            Journal = rt.models.ledger.Journal
+            VoucherTypes = rt.models.ledger.VoucherTypes
+            TradeTypes = rt.models.ledger.TradeTypes
+            sub = Subscription.get_by_ref(obj.site.ref, None)
+            if sub is None:
+                jnl = Journal.get_by_ref('SLA', None)
+                if jnl is None:
+                    vt = VoucherTypes.get_for_model(Subscription)
+                    jnl = Journal(ref="SLA", voucher_type=vt,
+                        trade_type=TradeTypes.sales,
+                        journal_group="sales",
+                        **dd.str2kw("name", _("Service Level Agreements")))
+                    jnl.full_clean()
+                    jnl.save()
+                sub = Subscription(partner=obj.site.company, jnl=jnl, ref=obj.site.ref)
+                sub.full_clean()
+                sub.save()
+            obj.order = sub
+            obj.full_clean()
+            obj.save()
+            logger.info("Filled %s order field after upgrade to Noi 22.12", obj)
+
+TicketOrderChecker.activate()
