@@ -1,8 +1,8 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2014-2022 Rumma & Ko Ltd
+# Copyright 2014-2023 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-# $ python setup.py test -s tests.test_packages
+# python setup.py test -s tests.test_packages
 
 ATELIER_INFO = dict(
     nickname="noi",
@@ -14,12 +14,11 @@ ATELIER_INFO = dict(
 
 SETUP_INFO = dict(
     name='lino-noi',
-    version='22.11.1',
+    version='23.1.0',
     install_requires=['lino-xl'],
     # tests_require=['pytest', 'mock'],
     test_suite='tests',
-    description=("A Lino Django application for managing "
-                 "support tickets and working time."),
+    description=("Manage support tickets and working time."),
     long_description="""\
 
 Lino Noi is a customizable ticket management and time tracking
@@ -75,6 +74,7 @@ lino_noi.lib.topics
 lino_noi.lib.users
 lino_noi.lib.users.fixtures
 lino_noi.lib.products
+lino_noi.lib.subscriptions
 lino_noi.lib.groups
 lino_noi.lib.cal
 lino_noi.lib.cal.fixtures
